import AppConfig from "./appconfig";
import { TokenGetter } from "./interface";
import { AxiosInstance } from "axios";
/**
 * AggregationClient - A client for making aggregation requests to Daptin backend
 */
declare class AggregationClient {
    private appConfig;
    private request;
    private tokenGetter;
    private axiosInstance;
    /**
     * Create a new aggregation client
     * @param appConfig
     * @param tokenGetter
     * @param axiosInstance
     */
    constructor(appConfig: AppConfig, tokenGetter: TokenGetter, axiosInstance: AxiosInstance);
    /**
     * Reset the current request to its initial state
     * @private
     */
    _resetRequest(): void;
    /**
     * Set the root entity for the aggregation
     * @param {string} entityName - The name of the entity to query
     * @returns {AggregationClient} - Returns this client for chaining
     */
    entity(entityName: any): this;
    /**
     * Add a join to the aggregation
     * @param {string} tableName - The name of the table to join
     * @param {string|Array<string>} conditions - Join conditions in the format "column eq value" or "column1 eq column2@table"
     * @returns {AggregationClient} - Returns this client for chaining
     */
    join(tableName: any, conditions: any): this;
    /**
     * Add grouping columns to the aggregation
     * @param {...string} columns - Columns to group by
     * @returns {AggregationClient} - Returns this client for chaining
     */
    groupBy(...columns: any[]): this;
    /**
     * Add projection columns to the aggregation
     * @param {...string} columns - Columns to project (can include aggregation functions like "count(*)", "sum(amount)", etc.)
     * @returns {AggregationClient} - Returns this client for chaining
     */
    project(...columns: any[]): this;
    /**
     * Add count to the projection
     * @returns {AggregationClient} - Returns this client for chaining
     */
    count(): this;
    /**
     * Add sum aggregation to the projection
     * @param {string} column - Column to sum
     * @returns {AggregationClient} - Returns this client for chaining
     */
    sum(column: any): this;
    /**
     * Add average aggregation to the projection
     * @param {string} column - Column to average
     * @returns {AggregationClient} - Returns this client for chaining
     */
    avg(column: any): this;
    /**
     * Add min aggregation to the projection
     * @param {string} column - Column to find minimum value
     * @returns {AggregationClient} - Returns this client for chaining
     */
    min(column: any): this;
    /**
     * Add max aggregation to the projection
     * @param {string} column - Column to find maximum value
     * @returns {AggregationClient} - Returns this client for chaining
     */
    max(column: any): this;
    /**
     * Add filter conditions
     * @param {string} column - Column name to filter on
     * @param {string} operator - Operator (eq, lt, gt, lte, gte, in, notin, is, not)
     * @param {string|number|boolean} value - Value to compare against
     * @returns {AggregationClient} - Returns this client for chaining
     */
    filter(column: any, operator: any, value: any): this;
    /**
     * Add equals filter
     * @param {string} column - Column name
     * @param {*} value - Value to compare
     * @returns {AggregationClient} - Returns this client for chaining
     */
    eq(column: any, value: any): this;
    /**
     * Add not equals filter
     * @param {string} column - Column name
     * @param {*} value - Value to compare
     * @returns {AggregationClient} - Returns this client for chaining
     */
    neq(column: any, value: any): this;
    /**
     * Add less than filter
     * @param {string} column - Column name
     * @param {*} value - Value to compare
     * @returns {AggregationClient} - Returns this client for chaining
     */
    lt(column: any, value: any): this;
    /**
     * Add less than or equal filter
     * @param {string} column - Column name
     * @param {*} value - Value to compare
     * @returns {AggregationClient} - Returns this client for chaining
     */
    lte(column: any, value: any): this;
    /**
     * Add greater than filter
     * @param {string} column - Column name
     * @param {*} value - Value to compare
     * @returns {AggregationClient} - Returns this client for chaining
     */
    gt(column: any, value: any): this;
    /**
     * Add greater than or equal filter
     * @param {string} column - Column name
     * @param {*} value - Value to compare
     * @returns {AggregationClient} - Returns this client for chaining
     */
    gte(column: any, value: any): this;
    /**
     * Add in filter (value in list)
     * @param {string} column - Column name
     * @param {Array} values - Values to check against
     * @returns {AggregationClient} - Returns this client for chaining
     */
    in(column: any, values: any): this;
    /**
     * Add not in filter (value not in list)
     * @param {string} column - Column name
     * @param {Array} values - Values to check against
     * @returns {AggregationClient} - Returns this client for chaining
     */
    notIn(column: any, values: any): this;
    /**
     * Add is null filter
     * @param {string} column - Column name
     * @returns {AggregationClient} - Returns this client for chaining
     */
    isNull(column: any): this;
    /**
     * Add is not null filter
     * @param {string} column - Column name
     * @returns {AggregationClient} - Returns this client for chaining
     */
    isNotNull(column: any): this;
    /**
     * Add is true filter
     * @param {string} column - Column name
     * @returns {AggregationClient} - Returns this client for chaining
     */
    isTrue(column: any): this;
    /**
     * Add is not true filter
     * @param {string} column - Column name
     * @returns {AggregationClient} - Returns this client for chaining
     */
    isNotTrue(column: any): this;
    /**
     * Add is false filter
     * @param {string} column - Column name
     * @returns {AggregationClient} - Returns this client for chaining
     */
    isFalse(column: any): this;
    /**
     * Add is not false filter
     * @param {string} column - Column name
     * @returns {AggregationClient} - Returns this client for chaining
     */
    isNotFalse(column: any): this;
    /**
     * Add having conditions for filtering aggregate results
     * @param {string} aggregateExpr - Aggregate expression (e.g., "count(*)", "sum(amount)")
     * @param {string} operator - Operator (eq, lt, gt, lte, gte)
     * @param {*} value - Value to compare against
     * @returns {AggregationClient} - Returns this client for chaining
     */
    having(aggregateExpr: any, operator: any, value: any): this;
    /**
     * Add order by clause
     * @param {...string} columns - Columns to order by (prefix with '-' for descending order)
     * @returns {AggregationClient} - Returns this client for chaining
     */
    orderBy(...columns: any[]): this;
    /**
     * Set time sample format (for time series data)
     * @param {string} format - Time sample format
     * @returns {AggregationClient} - Returns this client for chaining
     */
    timeSample(format: any): this;
    /**
     * Set time range from
     * @param {string} timeFrom - Start time
     * @returns {AggregationClient} - Returns this client for chaining
     */
    timeFrom(timeFrom: any): this;
    /**
     * Set time range to
     * @param {string} timeTo - End time
     * @returns {AggregationClient} - Returns this client for chaining
     */
    timeTo(timeTo: any): this;
    /**
     * Execute the aggregation request
     * @param {Object} options - Additional options for the request
     * @param {boolean} options.rawResponse - If true, returns the raw response instead of just the data
     * @returns {Promise<Object>} - Promise resolving to the aggregation results
     */
    execute(options?: {
        rawResponse: boolean;
    }): Promise<any>;
    /**
     * Create a reference object for use in filters
     * @param {string} entity - Entity name
     * @param {string} id - Entity ID (UUID)
     * @returns {Object} - Reference object
     */
    static ref(entity: any, id: any): {
        entity: any;
        id: any;
    };
}
export default AggregationClient;
